﻿/**
* Sample code for the CreateShipment Canada Post service.
* 
* The CreateShipment service is used to initiate generation of a shipping label
* by providing shipment details. Use of this service indicates an intention to 
* pay for shipment of an item.
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace CreateShipment
{
    class CreateShipment
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_Shipping_DotNet_Samples\SOAP\shipping\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            ShipmentPortTypeClient server = new ShipmentPortTypeClient("ShipmentPort", "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            createshipmentrequest request = new createshipmentrequest();
            request.locale = "FR";
            request.mailedby = mailedBy;

            // Create shipment object            
            ShipmentType shipment = new ShipmentType();
            request.shipment = new ShipmentType();
            request.shipment = shipment;
            shipment.deliveryspec = new DeliverySpecType();
            shipment.deliveryspec.sender = new SenderType();
            shipment.deliveryspec.sender.addressdetails = new AddressDetailsType();
            shipment.deliveryspec.destination = new DestinationType();
            shipment.deliveryspec.destination.addressdetails = new DestinationAddressDetailsType();
            shipment.deliveryspec.options = new OptionType[1];
            shipment.deliveryspec.options[0] = new OptionType();
            shipment.deliveryspec.parcelcharacteristics = new ParcelCharacteristicsType();
            shipment.deliveryspec.parcelcharacteristics.dimensions = new ParcelCharacteristicsTypeDimensions();
            shipment.deliveryspec.notification = new NotificationType();
            shipment.deliveryspec.printpreferences = new PrintPreferencesType();
            shipment.deliveryspec.preferences = new PreferencesType();
            shipment.deliveryspec.settlementinfo = new SettlementInfoType();
            shipment.deliveryspec.references = new ReferencesType();

            // Populate shipment object
            
            //1. regular shipment
            shipment.Item = "123456";

            //2. no manifest shipment
            //shipment.Item = true;
            
            shipment.requestedshippingpoint = "K1K4T3";
            shipment.cpcpickupindicator = true;
            shipment.cpcpickupindicatorSpecified = true;
            shipment.expectedmailingdate = new DateTime(2014, 08, 24);
            shipment.expectedmailingdateSpecified = true;
            shipment.deliveryspec.sender.addressdetails.addressline1 = "502 MAIN ST N";
            shipment.deliveryspec.sender.addressdetails.city = "MONTREAL";
            shipment.deliveryspec.sender.addressdetails.provstate = "QC";
            shipment.deliveryspec.sender.addressdetails.countrycode = "CA";
            shipment.deliveryspec.sender.addressdetails.postalzipcode = "H2B1A0";
            shipment.deliveryspec.sender.name = "Bulma";
            shipment.deliveryspec.sender.company = "Capsule Corp.";
            shipment.deliveryspec.sender.contactphone = "1 (514) 820 5879";
            shipment.deliveryspec.destination.addressdetails.addressline1 = "123 Postal Drive";
            shipment.deliveryspec.destination.addressdetails.city = "Ottawa";
            shipment.deliveryspec.destination.addressdetails.provstate = "ON";
            shipment.deliveryspec.destination.addressdetails.countrycode = "CA";
            shipment.deliveryspec.destination.addressdetails.postalzipcode = "K1P5Z9";
            shipment.deliveryspec.destination.name = "John Doe";
            shipment.deliveryspec.destination.company = "ACME Corp.";
            shipment.deliveryspec.options[0].optioncode = "DC";
            shipment.deliveryspec.parcelcharacteristics.dimensions.height = 6;
            shipment.deliveryspec.parcelcharacteristics.dimensions.length = 12;
            shipment.deliveryspec.parcelcharacteristics.dimensions.width = 9;
            shipment.deliveryspec.parcelcharacteristics.weight = 5;
            shipment.deliveryspec.notification.email = "ryuko.saito@kubere.com";
            shipment.deliveryspec.notification.ondelivery = true;
            shipment.deliveryspec.notification.onexception = false;
            shipment.deliveryspec.notification.onshipment = true;
            shipment.deliveryspec.printpreferences.outputformat = PrintPreferencesTypeOutputformat.Item85x11;
            shipment.deliveryspec.printpreferences.outputformatSpecified = true;
            shipment.deliveryspec.preferences.showinsuredvalue = true;
            shipment.deliveryspec.preferences.showpackinginstructions = true;
            shipment.deliveryspec.preferences.showpostagerate = true;
            shipment.deliveryspec.settlementinfo.contractid = "0042708517";
            shipment.deliveryspec.settlementinfo.intendedmethodofpayment = "Account";
            shipment.deliveryspec.references.costcentre = "costcentre";
            shipment.deliveryspec.references.customerref1 = "customerref1";
            shipment.deliveryspec.references.customerref2 = "customerref2";
            shipment.deliveryspec.servicecode = "DOM.EP";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                createshipmentresponse response = server.CreateShipment(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(ShipmentInfoDetailType))
                {
                    ShipmentInfoDetailType shipmentInfo = (ShipmentInfoDetailType)response.Item;
                    responseAsString += "Shipment ID: " + shipmentInfo.shipmentid + "\r\n";
                    responseAsString += "Tracking Pin: " + shipmentInfo.trackingpin + "\r\n";
                    foreach (var artifact in shipmentInfo.artifacts)
                    {
                        responseAsString += "Artifact Id:  " + artifact.artifactid + "\r\n";
                        responseAsString += "Page Index:  " + artifact.pageindex + "\r\n\r\n";
                    }
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);
        }
    }
}
