﻿/**
* Sample code for the GetShipment Canada Post service.
* 
* The GetShipment service is used to retrieve information related to a previously
* created shipment. This may be useful for recovery from a communication or other
* error and/or reporting on all outstanding shipments. 
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetShipment
{
    class GetShipment
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_Shipping_DotNet_Samples\SOAP\shipping\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            ShipmentPortTypeClient server = new ShipmentPortTypeClient("ShipmentPort", "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getshipmentrequest request = new getshipmentrequest();
            request.locale = "FR";
            request.mailedby = mailedBy;
            request.shipmentid = "340531309186521749";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getshipmentresponse response = server.GetShipment(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(ShipmentInfoType))
                {
                    ShipmentInfoType shipmentInfo = (ShipmentInfoType)response.Item;
                    responseAsString += "Shipment ID: " + shipmentInfo.shipmentid + "\r\n";
                    responseAsString += "Tracking Pin: " + shipmentInfo.trackingpin + "\r\n";
                    foreach (var artifact in shipmentInfo.artifacts)
                    {
                        responseAsString += "Artifact Id:  " + artifact.artifactid + "\r\n\r\n";
                    }
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
