﻿/**
* Sample code for the VoidShipment Canada Post service.
* 
* The VoidShipment service is used to indicate to Canada Post that a printed label 
* has been spoiled or cancelled and that it will not be used and should not be 
* transmitted or billed.
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace VoidShipment
{
    class VoidShipment
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_Shipping_DotNet_Samples\SOAP\shipping\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            ShipmentPortTypeClient server = new ShipmentPortTypeClient("ShipmentPort", "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            voidshipmentrequest request = new voidshipmentrequest();
            request.locale = "FR";
            request.mailedby = mailedBy;
            request.shipmentid = "340531309186521749";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                voidshipmentresponse response = server.VoidShipment(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(Boolean))
                {
                    Boolean voidShipment = (Boolean)response.Item;
                    if (voidShipment)
                    {
                        responseAsString += "Shipment successfully deleted.\r\n";
                    }
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
